@@ -209,18 +209,20 @@ g_daemon_vfs_init (GDaemonVfs *vfs)
   vfs->to_uri_hash = g_hash_table_new (g_str_hash, g_str_equal);
   
   mappers = g_type_children (G_VFS_TYPE_URI_MAPPER, &n_mappers);
+
   for (i = 0; i < n_mappers; i++)
     {
+      int j;
       mapper = g_object_new (mappers[i], NULL);
 
       schemes = g_vfs_uri_mapper_get_handled_schemes (mapper);
 
-      for (i = 0; schemes != NULL && schemes[i] != NULL; i++)
-	g_hash_table_insert (vfs->from_uri_hash, (char *)schemes[i], mapper);
+      for (j = 0; schemes != NULL && schemes[j] != NULL; j++)
+	g_hash_table_insert (vfs->from_uri_hash, (char *)schemes[j], mapper);
       
       mount_types = g_vfs_uri_mapper_get_handled_mount_types (mapper);
-      for (i = 0; mount_types != NULL && mount_types[i] != NULL; i++)
-	g_hash_table_insert (vfs->to_uri_hash, (char *)mount_types[i], mapper);
+      for (j = 0; mount_types != NULL && mount_types[j] != NULL; j++)
+	g_hash_table_insert (vfs->to_uri_hash, (char *)mount_types[j], mapper);
     }
 
   /* The above should have ref:ed the modules anyway */
@@ -703,9 +705,7 @@ _g_daemon_vfs_get_mount_info_sync (GMountSpec *spec,
 
   if (!reply)
     {
-      g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
-		   "Error while getting mount info: %s",
-		   derror.message);
+      _g_error_from_dbus (&derror, error);
       dbus_error_free (&derror);
       return NULL;
     }
@@ -769,9 +769,6 @@ g_daemon_vfs_class_init (GDaemonVfsClass *class)
 
   vfs_class = G_VFS_CLASS (class);
 
-  vfs_class->name = "gvfs";
-  vfs_class->priority = 10;
-  
   vfs_class->is_active = g_daemon_vfs_is_active;
   vfs_class->get_file_for_path = g_daemon_vfs_get_file_for_path;
   vfs_class->get_file_for_uri = g_daemon_vfs_get_file_for_uri;
@@ -782,6 +779,8 @@ g_daemon_vfs_class_init (GDaemonVfsClass *class)
 /* Module API */
 
 void g_vfs_uri_mapper_smb_register (GIOModule *module);
+void g_vfs_uri_mapper_http_register (GIOModule *module);
+void g_vfs_uri_mapper_sftp_register (GIOModule *module);
 
 void
 g_io_module_load (GIOModule *module)
@@ -789,8 +788,15 @@ g_io_module_load (GIOModule *module)
   g_daemon_vfs_register_type (G_TYPE_MODULE (module));
   g_daemon_volume_monitor_register_types (G_TYPE_MODULE (module));
 
+  g_io_extension_point_implement (G_VFS_EXTENSION_POINT_NAME,
+				  G_TYPE_DAEMON_VFS,
+				  "gvfs",
+				  10);
+  
   g_vfs_uri_mapper_register (module);
   g_vfs_uri_mapper_smb_register (module);
+  g_vfs_uri_mapper_sftp_register (module);
+  g_vfs_uri_mapper_http_register (module);
 }
 
 void
