@@ -27,8 +27,7 @@
 
 #include <glib/gi18n-lib.h>
 #include <gdbusutils.h>
-#include <gio/gcancellable.h>
-#include <gio/gioerror.h>
+#include <gio/gio.h>
 
 void
 _g_dbus_oom (void)
@@ -1070,7 +1069,7 @@ wakeup_main (void *data)
 static const GSourceFuncs dbus_source_funcs = {
   dbus_source_prepare,
   dbus_source_check,
-  dbus_source_dispatch,
+  dbus_source_dispatch
 };
 
 /* Called when the connection dies or when we're unintegrating from mainloop */
