@@ -418,6 +418,7 @@ recompute_files (GVfsBackendComputer *backend)
           file->icon = g_volume_get_icon (file->volume);
           file->display_name = g_volume_get_name (file->volume);
           file->can_mount = g_volume_can_mount (file->volume);
+          file->root = NULL;
           file->can_eject = g_volume_can_eject (file->volume);
         }
       else /* drive */
@@ -637,7 +638,7 @@ try_enumerate (GVfsBackend *backend,
       if (file != NULL)
         g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                           G_IO_ERROR_NOT_DIRECTORY,
-                          _("Can't enumerate non-directory"));
+                          _("The file is not a directory"));
       return TRUE;
     }
 
@@ -686,6 +687,7 @@ try_query_info (GVfsBackend *backend,
       g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_WRITE, FALSE);
       g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_DELETE, FALSE);
       g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_ACCESS_CAN_TRASH, FALSE);
+      g_file_info_set_content_type (info, "inode/directory");
     }
   else if (file != NULL)
     file_info_from_file (file, info);
@@ -760,7 +762,7 @@ mount_volume_cb (GObject *source_object,
       else
         g_vfs_job_failed (G_VFS_JOB (job), G_IO_ERROR,
                           G_IO_ERROR_FAILED,
-                          _("Can't find mount for mounted volume"));
+                          _("Internal error: %s"), "No mount object for mounted volume");
     }
   else
     {
