@@ -35,8 +35,6 @@
 #include <gio/gio.h>
 
 #include <libsoup/soup.h>
-#include <libsoup/soup-headers.h>
-#include <libsoup/soup-uri.h>
 
 /* LibXML2 includes */
 #include <libxml/parser.h>
@@ -66,9 +64,6 @@ struct _GVfsBackendDav
 {
   GVfsBackendHttp parent_instance;
 
-  /* Only used during mount: */
-  char         *last_good_path;
-  GMountSource *mount_source;
 };
 
 G_DEFINE_TYPE (GVfsBackendDav, g_vfs_backend_dav, G_VFS_TYPE_BACKEND_HTTP);
@@ -95,28 +90,14 @@ g_vfs_backend_dav_init (GVfsBackendDav *backend)
 static inline gboolean
 sm_has_header (SoupMessage *msg, const char *header)
 {
-  return soup_message_get_header (msg->response_headers, header) != NULL;
+  return soup_message_headers_get (msg->response_headers, header) != NULL;
 }
 
-static inline guint
-soup_protocol_default_port (SoupProtocol proto)
-{
-	if (proto == SOUP_PROTOCOL_HTTP)
-		return 80;
-	else if (proto == SOUP_PROTOCOL_HTTPS)
-		return 443;
-	else
-		return 0;
-}
-
-/* ************************************************************************* */
-/*  */
-
 static inline void
-send_message (GVfsBackend           *backend,
-              SoupMessage           *message, 
-              SoupMessageCallbackFn  callback,
-              gpointer               user_data)
+send_message (GVfsBackend         *backend,
+              SoupMessage         *message, 
+              SoupSessionCallback  callback,
+              gpointer             user_data)
 {
 
   soup_session_queue_message (G_VFS_BACKEND_HTTP (backend)->session,
@@ -124,29 +105,48 @@ send_message (GVfsBackend           *backend,
                               callback, user_data);
 }
 
-static inline SoupMessage *
-message_new_from_filename (GVfsBackend *backend,
-                           const char  *method,
-                           const char  *filename)
+static char *
+path_get_parent_dir (const char *path)
 {
-  SoupMessage *msg;
-  SoupUri     *uri;
+    char   *parent;
+    size_t  len;
 
-  uri = g_vfs_backend_uri_for_filename (backend, filename);
-  msg = soup_message_new_from_uri (method, uri);
-  soup_uri_free (uri);
+    if ((len = strlen (path)) < 1)
+      return NULL;
 
-  soup_message_add_header (msg->request_headers,
-                           "User-Agent", "gvfs/" VERSION);
-  return msg;
+    /* maybe this should be while, but then again
+     * I should be reading the uri rfc and see 
+     * what the deal was with multiple slashes */
+
+    if (path[len - 1] == '/')
+      len--;
+
+    parent = g_strrstr_len (path, len, "/");
+
+    if (parent == NULL)
+      return NULL;
+
+    return g_strndup (path, (parent - path) + 1);
 }
 
 /* ************************************************************************* */
 /*  */
-static xmlDocPtr
-multistatus_parse_xml (SoupMessage *msg, xmlNodePtr *root, GError **error)
+
+static inline gboolean
+node_has_name (xmlNodePtr node, const char *name)
 {
-  xmlDocPtr  doc;
+  g_return_val_if_fail (node != NULL, FALSE);
+
+  return ! strcmp ((char *) node->name, name);
+}
+
+static xmlDocPtr
+parse_xml (SoupMessage  *msg,
+           xmlNodePtr   *root,
+           const char   *name,
+           GError      **error)
+{
+ xmlDocPtr  doc;
 
   if (!SOUP_STATUS_IS_SUCCESSFUL (msg->status_code))
     {
@@ -156,8 +156,8 @@ multistatus_parse_xml (SoupMessage *msg, xmlNodePtr *root, GError **error)
       return NULL;
     }
 
-  doc = xmlReadMemory (msg->response.body,
-                       msg->response.length,
+  doc = xmlReadMemory (msg->response_body->data,
+                       msg->response_body->length,
                        "response.xml",
                        NULL,
                        0);
@@ -177,7 +177,7 @@ multistatus_parse_xml (SoupMessage *msg, xmlNodePtr *root, GError **error)
       return NULL;
     }
 
-  if (strcmp ((char *) (*root)->name, "multistatus"))
+  if (strcmp ((char *) (*root)->name, name))
     {
         g_set_error (error, G_IO_ERROR, G_IO_ERROR_FAILED,
                      _("Unexpected reply from server"));
@@ -187,6 +187,14 @@ multistatus_parse_xml (SoupMessage *msg, xmlNodePtr *root, GError **error)
   return doc;
 }
 
+
+static xmlDocPtr
+multistatus_parse_xml (SoupMessage *msg, xmlNodePtr *root, GError **error)
+{
+    return parse_xml (msg, root, "multistatus", error);
+}
+
+
 static GFileType
 parse_resourcetype (xmlNodePtr rt)
 {
@@ -213,57 +221,80 @@ parse_resourcetype (xmlNodePtr rt)
   return type;
 }
 
+static const char *
+node_get_content (xmlNodePtr node)
+{
+    if (node == NULL)
+      return NULL;
+
+    switch (node->type)
+      {
+        case XML_ELEMENT_NODE:
+          return node_get_content (node->children);
+          break;
+        case XML_TEXT_NODE:
+          return (const char *) node->content;
+          break;
+        default:
+          return NULL;
+      }
+    return NULL;
+}
+
 static GFileInfo *
 mulitstatus_parse_prop_node (xmlDocPtr doc, xmlNodePtr prop)
 {
   GFileInfo  *info;
   xmlNodePtr  node;
+  const char *text;
+  GTimeVal    tv;
 
   info = g_file_info_new ();
 
+
   for (node = prop->children; node; node = node->next)
     {
      if (node->type != XML_ELEMENT_NODE ||
-          node->name == NULL)
+         node->name == NULL)
         {
           continue;
         }
-     else if (!strcmp ((char *) node->name, "resourcetype"))
+
+     text = node_get_content (node);
+
+     if (node_has_name (node, "resourcetype"))
         {
            GFileType type = parse_resourcetype (node);
            g_file_info_set_file_type (info, type);
         }
-     else if (!strcmp ((char *) node->name, "displayname"))
+     else if (node_has_name (node, "displayname"))
         {
-          xmlChar *text;
-          text = xmlNodeGetContent (node);
-          g_file_info_set_display_name (info, (char *) text);
-          xmlFree (text);
+          g_file_info_set_display_name (info, text);
         }
-     else if (!strcmp ((char *) node->name, "getetag"))
+     else if (node_has_name (node, "getetag"))
         {
-
+          g_file_info_set_attribute_string (info, G_FILE_ATTRIBUTE_ETAG_VALUE,
+                                            text);
         }
-     else if (!strcmp ((char *) node->name, "creationdate"))
+     else if (node_has_name (node, "creationdate"))
         {
-          
+            
         }
-     else if (!strcmp ((char *) node->name, "getcontenttype"))
+     else if (node_has_name (node, "getcontenttype"))
         {
-          xmlChar *text;
-          text = xmlNodeGetContent (node);
-          g_file_info_set_content_type (info, (char *) text);
-          xmlFree (text);
+          g_file_info_set_content_type (info, text);
         }
-     else if (!strcmp ((char *) node->name, "getcontentlength"))
+     else if (node_has_name (node, "getcontentlength"))
         {
-          gint64 size; 
-          xmlChar *text;
-          text = xmlNodeGetContent (node);
-          size = g_ascii_strtoll ((char *) text, NULL, 10);
-          xmlFree (text);
+          gint64 size;
+          size = g_ascii_strtoll (text, NULL, 10);
           g_file_info_set_size (info, size);
         }
+     else if (node_has_name (node, "getlastmodified"))
+       {
+           if (g_time_val_from_iso8601 (text, &tv))
+             g_file_info_set_modification_time (info, &tv);
+       }
     }
   return info;
 }
@@ -271,7 +302,7 @@ mulitstatus_parse_prop_node (xmlDocPtr doc, xmlNodePtr prop)
 static GFileInfo *
 multistatus_parse_response (xmlDocPtr    doc,
                             xmlNodePtr   resp,
-                            SoupUri     *base)
+                            SoupURI     *base)
 {
   GFileInfo  *info;
   xmlNodePtr  node;
@@ -288,28 +319,15 @@ multistatus_parse_response (xmlDocPtr    doc,
         {
           continue;
         }
-      else if (! strcmp ((char *) node->name, "href"))
+      else if (node_has_name (node, "href"))
         {
-          xmlChar *text;
-          char    *path;
-          char    *p;
-          size_t   len;
+          const char *text;
 
-          /* FIXME: redo this (quickly hacked together) */
-          text = xmlNodeGetContent (node);
-          path = (char *) text;
-
-          if ((len = strlen (path)) > 1)
-            {
-              if (path[len - 1] == '/')
-                len--;
-            }
-
-          p = g_strrstr_len (path, len, "/");
-
-          name = g_strndup (p + 1, (len - (p - path + 1)));
+          text = node_get_content (node); 
+          name = uri_get_basename (text);
         }
-      else if (! strcmp ((char *) node->name, "propstat"))
+
+      else if (node_has_name (node, "propstat"))
         {
           xmlNodePtr  iter;
           xmlNodePtr  prop;
@@ -324,11 +342,11 @@ multistatus_parse_response (xmlDocPtr    doc,
                 if (node->type != XML_ELEMENT_NODE ||
                     node->name == NULL)
                     continue;
-                else if (! strcmp ((char *) iter->name, "status"))
+                else if (node_has_name (iter, "status"))
                   {
                     status_text = xmlNodeGetContent (iter);
                   }
-                else if (! strcmp ((char *) iter->name, "prop"))
+                else if (node_has_name (iter, "prop"))
                   {
                     prop = iter;
                   }
@@ -341,7 +359,6 @@ multistatus_parse_response (xmlDocPtr    doc,
             {
               if (status_text)
                 xmlFree (status_text);
-
               continue;
             } 
 
@@ -363,6 +380,7 @@ multistatus_parse_response (xmlDocPtr    doc,
   
   if (info && name)
     {
+      /* FIXME: that is *not* the name, since its not relative */
       g_file_info_set_name (info, name);
       g_file_info_set_edit_name (info, name);
     }
@@ -418,136 +436,178 @@ create_propfind_request (GFileAttributeMatcher *matcher, gulong *size)
     return res;
 }
 
+
 /* ************************************************************************* */
 /*  */
 
+typedef struct _MountOpData {
+
+  gulong        signal_id;
+
+  SoupSession  *session;
+  GMountSource *mount_source;
+
+  char         *last_good_path;
+
+  /* general authentication */
+
+  SoupMessage *message;
+  SoupAuth    *auth;
+
+     /* for server authentication */
+  char *username;
+  char *password;
+  char *last_realm;
+
+     /* for proxy authentication */
+  char *proxy_user;
+  char *proxy_password;
+
+} MountOpData;
+
 static void
-do_authentication (SoupSession *session,
+mount_op_data_free (gpointer _data)
+{
+  MountOpData *data;
+
+  data = (MountOpData *) _data;
+  
+  g_free (data->last_good_path);
+  
+  if (data->mount_source)
+    g_object_unref (data->mount_source);
+  
+  if (data->message)
+    g_object_unref (data->message);
+  
+  if (data->auth)
+    g_object_unref (data->auth);
+  
+  g_free (data->username);
+  g_free (data->password);
+  g_free (data->last_realm);
+  g_free (data->proxy_user);
+  g_free (data->proxy_password);
+
+  g_free (data);
+}
+
+static void
+ask_password_ready (GObject      *source_object,
+                    GAsyncResult *result,
+                    gpointer      user_data)
+{
+  MountOpData *data;
+  char        *username;
+  char        *password;
+  gboolean     aborted;
+  gboolean     res;
+
+  data = (MountOpData *) user_data;
+  username = password = NULL;
+
+  res = g_mount_source_ask_password_finish (data->mount_source,
+                                            result,
+                                            &aborted,
+                                            &password,
+                                            &username,
+                                            NULL);
+
+  if (res && !aborted)
+    soup_auth_authenticate (data->auth, username, password);
+
+  soup_session_unpause_message (data->session, data->message);
+
+  g_free (username);
+  g_free (password);
+
+  g_object_unref (data->message);
+  g_object_unref (data->auth);
+  
+  data->message = NULL;
+  data->auth = NULL;
+}
+
+static void
+soup_authenticate (SoupSession *session,
                    SoupMessage *msg,
-                   gchar       *auth_type,
-                   gchar       *auth_realm,
-                   char       **username,
-                   char       **password,
-                   gboolean     is_reprompt,
+	           SoupAuth    *auth,
+                   gboolean     retrying,
                    gpointer     user_data)
 {
-  GVfsBackendDav *backend;
-  GVfsJobMount   *job;
-  SoupUri        *mount_base;
-  gboolean        res;
-  gboolean        aborted;
+  MountOpData    *data;
+  const char     *username;
+  const char     *password;
   char           *prompt;
-  char           *new_password;
-  char           *new_user;
 
-  job = G_VFS_JOB_MOUNT (user_data);
-  backend = G_VFS_BACKEND_DAV (job->backend);
-  mount_base = G_VFS_BACKEND_HTTP (backend)->mount_base;
+  data = (MountOpData *) user_data;
 
-  new_user = new_password = NULL;
-  prompt = g_strdup_printf (_("Enter password for %s"), auth_realm);
+  if (soup_auth_is_for_proxy (auth))
+    {
+      username = data->proxy_user;
+      password = retrying ? NULL : data->proxy_password;
+    }
+  else
+    {
+      username = data->username;
+      password = retrying ? NULL : data->password;
+    }
 
-  g_print ("+ authenticate \n");
+  if (username && password)
+    {
+      soup_auth_authenticate (auth, username, password);
+      return;
+    }
 
-  res = g_mount_source_ask_password (backend->mount_source,
+  if (soup_auth_is_for_proxy (auth))
+    {
+      prompt = g_strdup (_("Please enter proxy password"));
+    }
+  else
+    {
+      const char *auth_realm;
+
+      auth_realm = soup_auth_get_realm (auth);
+
+      if (auth_realm == NULL)
+        auth_realm = _("WebDAV share");
+
+      prompt = g_strdup_printf (_("Enter password for %s"), auth_realm);
+    }
+
+  data->auth = g_object_ref (auth);
+  data->message = g_object_ref (msg);
+
+  soup_session_pause_message (data->session, msg);
+
+  g_mount_source_ask_password_async (data->mount_source,
                                      prompt,
-                                     mount_base->user,
+                                     username,
                                      NULL,
                                      G_ASK_PASSWORD_NEED_PASSWORD |
                                      G_ASK_PASSWORD_NEED_USERNAME,
-                                     &aborted,
-                                     &new_password,
-                                     &new_user,
-                                     NULL);
-
-  if (res && !aborted)
-    {
-      g_print ("Setting u/p\n");
-      *username = new_user;
-      *password = new_password;
-    }
-  else
-   {
-     g_free (new_user);
-     g_free (new_password);
-   }
-
+                                     ask_password_ready,
+                                     data);
   g_free (prompt);
-
-  g_print ("- authenticate \n");
-}
-
-static void 
-authenticate (SoupSession *session,
-              SoupMessage *msg,
-              gchar       *auth_type,
-              gchar       *auth_realm,
-              char       **username,
-              char       **password,
-              gpointer     user_data)
-{
-  do_authentication (session,
-                     msg,
-                     auth_type,
-                     auth_realm,
-                     username,
-                     password,
-                     FALSE,
-                     user_data);
-}
-
-static void 
-reauthenticate (SoupSession *session,
-                SoupMessage *msg,
-                gchar       *auth_type,
-                gchar       *auth_realm,
-                gpointer     username,
-                gpointer     password,
-                gpointer     user_data)
-{
-  do_authentication (session,
-                     msg,
-                     auth_type,
-                     auth_realm,
-                     username,
-                     password,
-                     TRUE,
-                     user_data);
-}
-
-static void discover_mount_root_ready (SoupMessage *msg,
-                                       gpointer     user_data);
-static void
-discover_mount_root (GVfsBackendDav *backend, GVfsJobMount *job)
-{
-  GVfsBackendHttp *http_backend;
-  SoupMessage     *msg;
-  SoupSession     *session;
-  SoupUri         *mount_base;
-
-  http_backend = G_VFS_BACKEND_HTTP (backend);
-  mount_base = http_backend->mount_base;
-  session = http_backend->session;
-
-  msg = soup_message_new_from_uri (SOUP_METHOD_OPTIONS, mount_base);
-  soup_message_add_header (msg->request_headers, "User-Agent", "gvfs/" VERSION);
-  soup_session_queue_message (session, msg, discover_mount_root_ready, job);
 }
 
 static void
-discover_mount_root_ready (SoupMessage *msg,
+discover_mount_root_ready (SoupSession *session,
+                           SoupMessage *msg,
                            gpointer     user_data)
 {
   GVfsBackendDav *backend;
   GVfsJobMount   *job;
+  MountOpData    *data;
   GMountSpec     *mount_spec;
-  SoupUri        *mount_base;
+  SoupURI        *mount_base;
   gboolean        is_success;
   gboolean        is_dav;
 
   job = G_VFS_JOB_MOUNT (user_data);
   backend = G_VFS_BACKEND_DAV (job->backend);
   mount_base = G_VFS_BACKEND_HTTP (backend)->mount_base;
+  data = (MountOpData *) G_VFS_JOB (job)->backend_data;
 
   is_success = SOUP_STATUS_IS_SUCCESSFUL (msg->status_code);
   is_dav = sm_has_header (msg, "DAV");
@@ -556,30 +616,25 @@ discover_mount_root_ready (SoupMessage *msg,
 
   if (is_success && is_dav)
     {
-      char *parent, *path;
-      size_t len;
 
-      path = backend->last_good_path = mount_base->path;
-      mount_base->path = NULL;
+      data->last_good_path = mount_base->path;
+      mount_base->path = path_get_parent_dir (mount_base->path);
 
-      if ((len = strlen (path)) > 1)
+      if (mount_base->path)
         {
-          if (path[len - 1] == '/')
-           len--;
-
-          parent = g_strrstr_len (path, len, "/");
-          mount_base->path = g_strndup (path, (parent - path) + 1);
-          discover_mount_root (backend, job);
+          SoupMessage *msg;
+          msg = message_new_from_uri (SOUP_METHOD_OPTIONS, mount_base);
+          soup_session_queue_message (session, msg, 
+                                      discover_mount_root_ready, job);
           return;
-        }
-      
+        } 
     }
 
   /* we have reached the end of paths we are allowed to
-   * chdir up to */
+   * chdir up to (or couldn't chdir up at all) */
   
   /* check if we at all have a good path */
-  if (backend->last_good_path == NULL)
+  if (data->last_good_path == NULL)
     {
       if (!is_success) 
         g_vfs_job_failed (G_VFS_JOB (job),
@@ -593,57 +648,33 @@ discover_mount_root_ready (SoupMessage *msg,
       return;
     }
 
+  g_free (mount_base->path);
+  mount_base->path = data->last_good_path;
+  data->last_good_path = NULL;
   mount_spec = g_mount_spec_new ("dav"); 
+  
   g_mount_spec_set (mount_spec, "host", mount_base->host);
 
   if (mount_base->user)
     g_mount_spec_set (mount_spec, "user", mount_base->user);
 
-  if (mount_base->protocol == SOUP_PROTOCOL_HTTP)
+  if (mount_base->scheme == SOUP_URI_SCHEME_HTTP)
     g_mount_spec_set (mount_spec, "ssl", "false");
-  else if (mount_base->protocol == SOUP_PROTOCOL_HTTPS)
+  else if (mount_base->scheme == SOUP_URI_SCHEME_HTTPS)
     g_mount_spec_set (mount_spec, "ssl", "true");
 
-  g_free (mount_base->path);
-  mount_base->path = backend->last_good_path;
-
   g_mount_spec_set_mount_prefix (mount_spec, mount_base->path);
-  g_vfs_backend_set_mount_spec (G_VFS_BACKEND (backend), mount_spec);
   g_vfs_backend_set_icon_name (G_VFS_BACKEND (backend), "folder-remote");
 
+  g_vfs_backend_set_mount_spec (G_VFS_BACKEND (backend), mount_spec);
   g_mount_spec_unref (mount_spec);
+
+  g_signal_handler_disconnect (session, data->signal_id);
+
   g_print ("- discover_mount_root_ready success: %s \n", mount_base->path);
   g_vfs_job_succeeded (G_VFS_JOB (job));
 }
 
-
-
-static void
-mount (GVfsBackend  *backend,
-       GVfsJobMount *job,
-       GMountSpec   *mount_spec,
-       GMountSource *mount_source,
-       gboolean      is_automount)
-{
-  GVfsBackendDav *op_backend;
-  SoupSession    *session;
-
-  g_print ("+ mount\n");
-
-  op_backend = G_VFS_BACKEND_DAV (backend);
-  session = G_VFS_BACKEND_HTTP (backend)->session;
-
-  g_signal_connect (session, "authenticate",
-                    G_CALLBACK (authenticate), job);
-
-  g_signal_connect (session, "reauthenticate",
-                    G_CALLBACK (reauthenticate), job);
-
-  op_backend->mount_source = mount_source;
-  discover_mount_root (op_backend, job);
-  g_print ("- mount\n");
-}
-
 static gboolean
 try_mount (GVfsBackend  *backend,
            GVfsJobMount *job,
@@ -652,18 +683,17 @@ try_mount (GVfsBackend  *backend,
            gboolean      is_automount)
 {
   GVfsBackendDav *op_backend;
-  SoupUri        *uri;
+  MountOpData    *data;
+  SoupSession    *session;
+  SoupMessage    *msg;
+  SoupURI        *uri;
   const char     *host;
   const char     *user;
   const char     *port;
   const char     *ssl;
   guint           port_num;
 
-  /* We have to override this method since the http backend
-   * has its own try_mount and we don't want that to be used.
-   * We also need to do the dav mounting stuff inside a
-   * thread since the auth callback from soup is getting
-   * called in the main thread otherwise and we would block */
+  g_print ("+ mount\n");
 
   op_backend = G_VFS_BACKEND_DAV (backend);
 
@@ -681,35 +711,49 @@ try_mount (GVfsBackend  *backend,
       return TRUE;
     }
 
-  uri = g_new0 (SoupUri, 1);
+  uri = soup_uri_new (NULL);
 
   if (ssl != NULL && (strcmp (ssl, "true") == 0))
-    uri->protocol = SOUP_PROTOCOL_HTTPS;
+    soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTPS);
   else
-    uri->protocol = SOUP_PROTOCOL_HTTP;
-  
-  uri->user = g_strdup (user);
+    soup_uri_set_scheme (uri, SOUP_URI_SCHEME_HTTP);
+
+  soup_uri_set_user (uri, user);
 
   if (port && (port_num = atoi (port)))
-    uri->port = port_num;
-  else
-    uri->port = soup_protocol_default_port (uri->protocol);
+    soup_uri_set_port (uri, port_num);
 
-  uri->host = g_strdup (host);
-  uri->path = g_strdup (mount_spec->mount_prefix);
+  soup_uri_set_host (uri, host);
+  soup_uri_set_path (uri, mount_spec->mount_prefix);
 
+  session = G_VFS_BACKEND_HTTP (backend)->session;
   G_VFS_BACKEND_HTTP (backend)->mount_base = uri; 
 
-  return FALSE;
+  msg = message_new_from_uri (SOUP_METHOD_OPTIONS, uri);
+  soup_session_queue_message (session, msg, discover_mount_root_ready, job);
+
+  data = g_new0 (MountOpData, 1);
+  data->session = g_object_ref (session);
+  data->mount_source = g_object_ref (mount_source);
+  data->username = g_strdup (user);
+
+  data->signal_id = g_signal_connect (session, "authenticate",
+                                      G_CALLBACK (soup_authenticate), data);
+
+  g_vfs_job_set_backend_data (G_VFS_JOB (job), data, mount_op_data_free);
+
+  g_print ("- mount\n");
+  return TRUE;
 }
 
 static void
-query_info_ready (SoupMessage *msg,
+query_info_ready (SoupSession *session,
+                  SoupMessage *msg,
                   gpointer     user_data)
 {
   GVfsBackendDav    *backend;
   GVfsJobQueryInfo  *job;
-  SoupUri           *base;
+  SoupURI           *base;
   GFileInfo         *info;
   GError            *error;
   xmlDocPtr          doc;
@@ -789,7 +833,7 @@ try_query_info (GVfsBackend           *backend,
       return TRUE;
     }
 
-  soup_message_add_header (msg->request_headers, "Depth", "0");
+  soup_message_headers_append (msg->request_headers, "Depth", "0");
 
   /* RFC 4437 */
   if (flags & G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS)
@@ -797,11 +841,11 @@ try_query_info (GVfsBackend           *backend,
   else
       redirect_header = "T";
 
-  soup_message_add_header (msg->request_headers,
+  soup_message_headers_append (msg->request_headers,
                            "Apply-To-Redirect-Ref", redirect_header);
 
   soup_message_set_request (msg, "application/xml",
-                            SOUP_BUFFER_SYSTEM_OWNED,
+                            SOUP_MEMORY_TAKE,
                             request,
                             len);
 
@@ -809,17 +853,18 @@ try_query_info (GVfsBackend           *backend,
   return TRUE;
 }
 
+
 /* *** enumerate *** */
 
 
-
 static void
-enumerate_ready (SoupMessage *msg,
+enumerate_ready (SoupSession *session,
+                 SoupMessage *msg,
                  gpointer     user_data)
 {
   GVfsBackendDav    *backend;
   GVfsJobEnumerate  *job;
-  SoupUri           *base;
+  SoupURI           *base;
   GFileInfo         *info;
   GError            *error;
   xmlDocPtr          doc;
@@ -903,7 +948,7 @@ try_enumerate (GVfsBackend           *backend,
       return TRUE;
     }
 
-  soup_message_add_header (msg->request_headers, "Depth", "1");
+  soup_message_headers_append (msg->request_headers, "Depth", "1");
 
   /* RFC 4437 */
   if (flags & G_FILE_QUERY_INFO_NOFOLLOW_SYMLINKS)
@@ -911,11 +956,11 @@ try_enumerate (GVfsBackend           *backend,
   else
       redirect_header = "T";
 
-  soup_message_add_header (msg->request_headers,
-                           "Apply-To-Redirect-Ref", redirect_header);
+  soup_message_headers_append (msg->request_headers,
+                               "Apply-To-Redirect-Ref", redirect_header);
 
   soup_message_set_request (msg, "application/xml",
-                            SOUP_BUFFER_SYSTEM_OWNED,
+                            SOUP_MEMORY_TAKE,
                             request,
                             len);
 
@@ -936,7 +981,6 @@ g_vfs_backend_dav_class_init (GVfsBackendDavClass *klass)
   gobject_class->finalize  = g_vfs_backend_dav_finalize;
 
   backend_class = G_VFS_BACKEND_CLASS (klass); 
-  backend_class->mount             = mount;
   backend_class->try_mount         = try_mount;
   backend_class->try_query_info    = try_query_info;
   backend_class->try_enumerate     = try_enumerate;
