@@ -679,6 +679,8 @@ do_mount (GVfsBackend *backend,
     }
   else
     {
+      /* translators: Name for the location that lists the smb shares
+	 availible on a server (%s is the name of the server) */
       display_name = g_strdup_printf (_("Windows shares on %s"), op_backend->server);
       browse_mount_spec = g_mount_spec_new ("smb-server");
       g_mount_spec_set (browse_mount_spec, "server", op_backend->server);
@@ -926,7 +928,7 @@ get_file_info_from_entry (GVfsBackendSmbBrowse *backend, BrowseEntry *entry, GFi
   g_file_info_set_name (info, entry->name);
   g_file_info_set_display_name (info, entry->name_utf8);
   g_file_info_set_edit_name (info, entry->name_utf8);
-  g_file_info_set_attribute_string (info, "smb:comment", entry->comment);
+  g_file_info_set_attribute_string (info, "smb::comment", entry->comment);
   g_file_info_set_attribute_boolean (info, G_FILE_ATTRIBUTE_STANDARD_IS_VIRTUAL, TRUE);
 
   icon = NULL;
