@@ -170,15 +170,18 @@ _drive_get_description (HalDevice *d)
       if (hal_device_get_property_bool (d, "storage.cdrom.bdre"))
         second = _("Blu-ray-RE");
       
-      if (second != NULL) {
-        /* translators: This wis something like "CD-ROM/DVD Drive" or
-           "CD-RW/Blue-ray Drive" depending on the properties of the drive */
-        s = g_strdup_printf (_("%s/%s Drive"), first, second);
-      } else {
-        /* translators: This wis something like "CD-ROM Drive" or "CD-RW Drive
-           depending on the properties of the drive */
-        s = g_strdup_printf (_("%s Drive"), first);
-      }
+      if (second != NULL)
+        {
+          /* translators: This wis something like "CD-ROM/DVD Drive" or
+             "CD-RW/Blue-ray Drive" depending on the properties of the drive */
+          s = g_strdup_printf (_("%s/%s Drive"), first, second);
+        }
+      else
+        {
+          /* translators: This wis something like "CD-ROM Drive" or "CD-RW Drive
+             depending on the properties of the drive */
+          s = g_strdup_printf (_("%s Drive"), first);
+        }
     } 
   else if (strcmp (drive_type, "floppy") == 0)
     s = g_strdup (_("Floppy Drive"));
@@ -357,8 +360,8 @@ hal_changed (HalDevice *device, const char *key, gpointer user_data)
 
 GHalDrive *
 g_hal_drive_new (GVolumeMonitor       *volume_monitor,
-                  HalDevice            *device,
-                  HalPool              *pool)
+                 HalDevice            *device,
+                 HalPool              *pool)
 {
   GHalDrive *drive;
 
@@ -840,6 +843,42 @@ g_hal_drive_poll_for_media_finish (GDrive        *drive,
   return TRUE;
 }
 
+static char *
+g_hal_drive_get_identifier (GDrive              *drive,
+                             const char          *kind)
+{
+  GHalDrive *hal_drive = G_HAL_DRIVE (drive);
+
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_HAL_UDI) == 0)
+    return g_strdup (hal_device_get_udi (hal_drive->device));
+  
+  if (strcmp (kind, G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE) == 0)
+    return g_strdup (hal_drive->device_path);
+  
+  return NULL;
+}
+
+static char **
+g_hal_drive_enumerate_identifiers (GDrive *drive)
+{
+  GHalDrive *hal_drive = G_HAL_DRIVE (drive);
+  GPtrArray *res;
+
+  res = g_ptr_array_new ();
+
+  g_ptr_array_add (res,
+                   g_strdup (G_VOLUME_IDENTIFIER_KIND_HAL_UDI));
+
+  if (hal_drive->device_path && *hal_drive->device_path != 0)
+    g_ptr_array_add (res,
+                     g_strdup (G_VOLUME_IDENTIFIER_KIND_UNIX_DEVICE));
+    
+
+  /* Null-terminate */
+  g_ptr_array_add (res, NULL);
+  
+  return g_ptr_array_free (res, FALSE);
+}
 
 static void
 g_hal_drive_drive_iface_init (GDriveIface *iface)
@@ -857,6 +896,8 @@ g_hal_drive_drive_iface_init (GDriveIface *iface)
   iface->eject_finish = g_hal_drive_eject_finish;
   iface->poll_for_media = g_hal_drive_poll_for_media;
   iface->poll_for_media_finish = g_hal_drive_poll_for_media_finish;
+  iface->get_identifier = g_hal_drive_get_identifier;
+  iface->enumerate_identifiers = g_hal_drive_enumerate_identifiers;
 }
 
 void 
