@@ -54,6 +54,7 @@ struct _GHalVolumeMonitor {
 
   HalPool *pool;
 
+  GList *last_camera_devices;;
   GList *last_optical_disc_devices;
   GList *last_drive_devices;
   GList *last_volume_devices;
@@ -67,6 +68,9 @@ struct _GHalVolumeMonitor {
   /* we keep volumes/mounts for blank and audio discs separate to handle e.g. mixed discs properly */
   GList *disc_volumes;
   GList *disc_mounts;
+
+  /* Digital cameras (e.g. gphoto2) are kept here */
+  GList *camera_volumes;
 };
 
 static void mountpoints_changed      (GUnixMountMonitor  *mount_monitor,
@@ -76,14 +80,27 @@ static void mounts_changed           (GUnixMountMonitor  *mount_monitor,
 static void hal_changed              (HalPool    *pool,
                                       HalDevice  *device,
                                       gpointer    user_data);
+static void update_all               (GHalVolumeMonitor *monitor);
 static void update_drives            (GHalVolumeMonitor *monitor);
 static void update_volumes           (GHalVolumeMonitor *monitor);
 static void update_mounts            (GHalVolumeMonitor *monitor);
 static void update_discs             (GHalVolumeMonitor *monitor);
+static void update_cameras           (GHalVolumeMonitor *monitor);
 
 #define g_hal_volume_monitor_get_type g_hal_volume_monitor_get_type
 G_DEFINE_DYNAMIC_TYPE (GHalVolumeMonitor, g_hal_volume_monitor, G_TYPE_NATIVE_VOLUME_MONITOR);
 
+static HalPool *
+get_hal_pool (void)
+{
+  char *cap_only[] = {"block", "camera", NULL};
+
+  if (pool == NULL)
+    pool = hal_pool_new (cap_only);
+  
+  return pool;
+}
+
 static void
 g_hal_volume_monitor_finalize (GObject *object)
 {
@@ -100,6 +117,8 @@ g_hal_volume_monitor_finalize (GObject *object)
   g_object_unref (monitor->mount_monitor);
   g_object_unref (monitor->pool);
 
+  g_list_foreach (monitor->last_camera_devices, (GFunc)g_object_unref, NULL);
+  g_list_free (monitor->last_optical_disc_devices);
   g_list_foreach (monitor->last_optical_disc_devices, (GFunc)g_object_unref, NULL);
   g_list_free (monitor->last_optical_disc_devices);
   g_list_foreach (monitor->last_drive_devices, (GFunc)g_object_unref, NULL);
@@ -122,6 +141,8 @@ g_hal_volume_monitor_finalize (GObject *object)
   g_list_free (monitor->disc_volumes);
   g_list_foreach (monitor->disc_mounts, (GFunc)g_object_unref, NULL);
   g_list_free (monitor->disc_mounts);
+  g_list_foreach (monitor->camera_volumes, (GFunc)g_object_unref, NULL);
+  g_list_free (monitor->camera_volumes);
   
   if (G_OBJECT_CLASS (g_hal_volume_monitor_parent_class)->finalize)
     (*G_OBJECT_CLASS (g_hal_volume_monitor_parent_class)->finalize) (object);
@@ -155,6 +176,8 @@ get_volumes (GVolumeMonitor *volume_monitor)
   l = g_list_copy (monitor->volumes);
   ll = g_list_copy (monitor->disc_volumes);
   l = g_list_concat (l, ll);
+  ll = g_list_copy (monitor->camera_volumes);
+  l = g_list_concat (l, ll);
 
   g_list_foreach (l, (GFunc)g_object_ref, NULL);
 
@@ -190,14 +213,14 @@ get_volume_for_uuid (GVolumeMonitor *volume_monitor, const char *uuid)
     {
       volume = l->data;
       if (g_hal_volume_has_uuid (volume, uuid))
-          goto found;
+        goto found;
     }
 
   for (l = monitor->disc_volumes; l != NULL; l = l->next)
     {
       volume = l->data;
       if (g_hal_volume_has_uuid (volume, uuid))
-          goto found;
+        goto found;
     }
 
   return NULL;
@@ -222,14 +245,14 @@ get_mount_for_uuid (GVolumeMonitor *volume_monitor, const char *uuid)
     {
       mount = l->data;
       if (g_hal_mount_has_uuid (mount, uuid))
-          goto found;
+        goto found;
     }
 
   for (l = monitor->disc_mounts; l != NULL; l = l->next)
     {
       mount = l->data;
       if (g_hal_mount_has_uuid (mount, uuid))
-          goto found;
+        goto found;
     }
 
   return NULL;
@@ -261,9 +284,7 @@ get_mount_for_mount_path (const char *mount_path,
       volume_monitor = G_HAL_VOLUME_MONITOR (g_hal_volume_monitor_new ());
     }
   else
-    {
-      volume_monitor = g_object_ref (the_volume_monitor);
-    }
+    volume_monitor = g_object_ref (the_volume_monitor);
 
   /* creation of the volume monitor might actually fail */
   if (volume_monitor != NULL)
@@ -293,10 +314,7 @@ mountpoints_changed (GUnixMountMonitor *mount_monitor,
 {
   GHalVolumeMonitor *monitor = G_HAL_VOLUME_MONITOR (user_data);
 
-  update_drives (monitor);
-  update_volumes (monitor);
-  update_mounts (monitor);
-  update_discs (monitor);
+  update_all (monitor);
 }
 
 static void
@@ -305,19 +323,13 @@ mounts_changed (GUnixMountMonitor *mount_monitor,
 {
   GHalVolumeMonitor *monitor = G_HAL_VOLUME_MONITOR (user_data);
 
-  update_drives (monitor);
-  update_volumes (monitor);
-  update_mounts (monitor);
-  update_discs (monitor);
+  update_all (monitor);
 }
 
 void 
 g_hal_volume_monitor_force_update (GHalVolumeMonitor *monitor)
 {
-  update_drives (monitor);
-  update_volumes (monitor);
-  update_mounts (monitor);
-  update_discs (monitor);
+  update_all (monitor);
 }
 
 static void
@@ -327,12 +339,9 @@ hal_changed (HalPool    *pool,
 {
   GHalVolumeMonitor *monitor = G_HAL_VOLUME_MONITOR (user_data);
   
-  //g_warning ("hal changed");
+  /*g_warning ("hal changed");*/
   
-  update_drives (monitor);
-  update_volumes (monitor);
-  update_mounts (monitor);
-  update_discs (monitor);
+  update_all (monitor);
 }
 
 static GObject *
@@ -351,13 +360,10 @@ g_hal_volume_monitor_constructor (GType                  type,
       return object;
     }
 
-  //g_warning ("creating hal vm");
+  /*g_warning ("creating hal vm");*/
 
   object = NULL;
 
-  /* Created by is_supported */
-  g_assert (pool != NULL);
-
   /* Invoke parent constructor. */
   klass = G_HAL_VOLUME_MONITOR_CLASS (g_type_class_peek (G_TYPE_HAL_VOLUME_MONITOR));
   parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
@@ -366,7 +372,7 @@ g_hal_volume_monitor_constructor (GType                  type,
                                       construct_properties);
 
   monitor = G_HAL_VOLUME_MONITOR (object);
-  monitor->pool = g_object_ref (pool);
+  monitor->pool = g_object_ref (get_hal_pool ());
 
   monitor->mount_monitor = g_unix_mount_monitor_new ();
 
@@ -386,10 +392,7 @@ g_hal_volume_monitor_constructor (GType                  type,
                     "device_removed", G_CALLBACK (hal_changed),
                     monitor);
 		    
-  update_drives (monitor);
-  update_volumes (monitor);
-  update_mounts (monitor);
-  update_discs (monitor);
+  update_all (monitor);
 
   the_volume_monitor = monitor;
 
@@ -414,8 +417,7 @@ g_hal_volume_monitor_class_finalize (GHalVolumeMonitorClass *klass)
 static gboolean
 is_supported (void)
 {
-  pool = hal_pool_new ("block");
-  return pool != NULL;
+  return get_hal_pool() != NULL;
 }
 
 static GVolume *
@@ -435,6 +437,7 @@ adopt_orphan_mount (GMount *mount)
   for (l = the_volume_monitor->disc_volumes; l != NULL; l = l->next)
     {
       GHalVolume *volume = l->data;
+      
       if (g_hal_volume_has_foreign_mount_root (volume, mount_root))
         {
           g_hal_volume_adopt_foreign_mount (volume, mount);
@@ -442,7 +445,7 @@ adopt_orphan_mount (GMount *mount)
           break;
         }
     }
-
+  
   g_object_unref (mount_root);
   return ret;
 }
@@ -531,7 +534,7 @@ diff_sorted_lists (GList         *list1,
 
 GHalVolume *
 g_hal_volume_monitor_lookup_volume_for_mount_path (GHalVolumeMonitor *monitor,
-                                                    const char         *mount_path)
+                                                   const char         *mount_path)
 {
   GList *l;
 
@@ -546,6 +549,23 @@ g_hal_volume_monitor_lookup_volume_for_mount_path (GHalVolumeMonitor *monitor,
   return NULL;
 }
 
+static GHalVolume *
+g_hal_volume_monitor_lookup_volume_for_device_path (GHalVolumeMonitor *monitor,
+                                                    const char         *device_path)
+{
+  GList *l;
+
+  for (l = monitor->volumes; l != NULL; l = l->next)
+    {
+      GHalVolume *volume = l->data;
+
+      if (g_hal_volume_has_device_path (volume, device_path))
+	return volume;
+    }
+  
+  return NULL;
+}
+
 
 
 static GHalMount *
@@ -605,7 +625,7 @@ find_disc_mount_by_udi (GHalVolumeMonitor *monitor, const char *udi)
   for (l = monitor->disc_mounts; l != NULL; l = l->next)
     {
       GHalMount *mount = l->data;
-
+      
       if (g_hal_mount_has_udi (mount, udi))
 	return mount;
     }
@@ -629,6 +649,24 @@ find_disc_volume_by_udi (GHalVolumeMonitor *monitor, const char *udi)
   return NULL;
 }
 
+#ifdef _WITH_GPHOTO2
+static GHalVolume *
+find_camera_volume_by_udi (GHalVolumeMonitor *monitor, const char *udi)
+{
+  GList *l;
+
+  for (l = monitor->camera_volumes; l != NULL; l = l->next)
+    {
+      GHalVolume *volume = l->data;
+
+      if (g_hal_volume_has_udi (volume, udi))
+	return volume;
+    }
+  
+  return NULL;
+}
+#endif
+
 static gint
 hal_device_compare (HalDevice *a, HalDevice *b)
 {
@@ -636,7 +674,7 @@ hal_device_compare (HalDevice *a, HalDevice *b)
 }
 
 static gboolean
-_should_volume_be_ignored (HalDevice *d)
+should_volume_be_ignored (HalPool *pool, HalDevice *d)
 {
   gboolean volume_ignore;
   const char *volume_fsusage;
@@ -646,7 +684,7 @@ _should_volume_be_ignored (HalDevice *d)
   
   if (volume_fsusage == NULL)
     {
-      //g_warning ("no volume.fsusage property. Refusing to ignore");
+      /*g_warning ("no volume.fsusage property. Refusing to ignore");*/
       return FALSE;
     }
 
@@ -656,6 +694,19 @@ _should_volume_be_ignored (HalDevice *d)
   if (strcmp (volume_fsusage, "filesystem") != 0)
     {
       /* no file system on the volume... blank and audio discs are handled in update_discs() */
+
+      /* TODO: davidz: LUKS stuff needs more work; turn off for now */
+#if 0
+      /* check if it's a LUKS crypto volume */
+      if (strcmp (volume_fsusage, "crypto") == 0)
+        {
+          if (strcmp (hal_device_get_property_string (d, "volume.fstype"), "crypto_LUKS") == 0)
+            {
+              return FALSE;
+            }
+        }
+#endif
+      
       return TRUE;
     }
 
@@ -668,15 +719,67 @@ _should_volume_be_ignored (HalDevice *d)
   return FALSE;
 }
 
+static gboolean
+should_drive_be_ignored (HalPool *pool, HalDevice *d)
+{
+  GList *volumes, *l;
+  const char *drive_udi;
+  gboolean all_volumes_ignored, got_volumes;
+
+  drive_udi = hal_device_get_udi (d);
+  
+  volumes = hal_pool_find_by_capability (pool, "volume");
+
+  all_volumes_ignored = TRUE;
+  got_volumes = FALSE;
+  for (l = volumes; l != NULL; l = l->next)
+    {
+      HalDevice *volume_dev = l->data;
+      if (strcmp (drive_udi, hal_device_get_property_string (volume_dev, "block.storage_device")) == 0)
+        {
+          got_volumes = TRUE;
+          if (!should_volume_be_ignored (pool, volume_dev))
+            {
+              all_volumes_ignored = FALSE;
+              break;
+            }
+        }
+    }
+
+  return got_volumes && all_volumes_ignored;
+}
+
+static void
+update_all (GHalVolumeMonitor *monitor)
+{
+  update_drives (monitor);
+  update_volumes (monitor);
+  update_mounts (monitor);
+  update_discs (monitor);
+  update_cameras (monitor);
+}
+
 static void
 update_drives (GHalVolumeMonitor *monitor)
 {
   GList *new_drive_devices;
   GList *removed, *added;
-  GList *l;
+  GList *l, *ll;
   GHalDrive *drive;
 
   new_drive_devices = hal_pool_find_by_capability (monitor->pool, "storage");
+
+  /* remove devices we want to ignore - we do it here so we get to reevaluate
+   * on the next update whether they should still be ignored
+   */
+  for (l = new_drive_devices; l != NULL; l = ll)
+    {
+      HalDevice *d = l->data;
+      ll = l->next;
+      if (should_drive_be_ignored (monitor->pool, d))
+        new_drive_devices = g_list_delete_link (new_drive_devices, l);
+    }
+
   g_list_foreach (new_drive_devices, (GFunc) g_object_ref, NULL);
 
   new_drive_devices = g_list_sort (new_drive_devices, (GCompareFunc) hal_device_compare);
@@ -691,7 +794,7 @@ update_drives (GHalVolumeMonitor *monitor)
       drive = find_drive_by_udi (monitor, hal_device_get_udi (d));
       if (drive != NULL)
         {
-          //g_warning ("hal removing drive %s", hal_device_get_property_string (d, "block.device"));
+          /*g_warning ("hal removing drive %s", hal_device_get_property_string (d, "block.device"));*/
           g_hal_drive_disconnected (drive);
           monitor->drives = g_list_remove (monitor->drives, drive);
           g_signal_emit_by_name (monitor, "drive_disconnected", drive);
@@ -706,7 +809,7 @@ update_drives (GHalVolumeMonitor *monitor)
       drive = find_drive_by_udi (monitor, hal_device_get_udi (d));
       if (drive == NULL)
         {
-          //g_warning ("hal adding drive %s", hal_device_get_property_string (d, "block.device"));
+          /*g_warning ("hal adding drive %s", hal_device_get_property_string (d, "block.device"));*/
           drive = g_hal_drive_new (G_VOLUME_MONITOR (monitor), d, monitor->pool);
           if (drive != NULL)
             {
@@ -739,9 +842,9 @@ update_volumes (GHalVolumeMonitor *monitor)
    */
   for (l = new_volume_devices; l != NULL; l = ll)
     {
-      ll = l->next;
       HalDevice *d = l->data;
-      if (_should_volume_be_ignored (d))
+      ll = l->next;
+      if (should_volume_be_ignored (monitor->pool, d))
         new_volume_devices = g_list_delete_link (new_volume_devices, l);
     }
 
@@ -759,7 +862,7 @@ update_volumes (GHalVolumeMonitor *monitor)
       volume = find_volume_by_udi (monitor, hal_device_get_udi (d));
       if (volume != NULL)
         {
-          //g_warning ("hal removing vol %s", hal_device_get_property_string (d, "block.device"));
+          /*g_warning ("hal removing vol %s", hal_device_get_property_string (d, "block.device"));*/
           g_hal_volume_removed (volume);
           monitor->volumes = g_list_remove (monitor->volumes, volume);
           g_signal_emit_by_name (monitor, "volume_removed", volume);
@@ -776,9 +879,9 @@ update_volumes (GHalVolumeMonitor *monitor)
       if (volume == NULL)
         {
           drive = find_drive_by_udi (monitor, hal_device_get_property_string (d, "block.storage_device"));
-
-          //g_warning ("hal adding vol %s (drive %p)", hal_device_get_property_string (d, "block.device"), drive);
-          volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, NULL, drive);
+          
+          /*g_warning ("hal adding vol %s (drive %p)", hal_device_get_property_string (d, "block.device"), drive);*/
+          volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, NULL, TRUE, drive);
           if (volume != NULL)
             {
               monitor->volumes = g_list_prepend (monitor->volumes, volume);
@@ -802,6 +905,7 @@ update_mounts (GHalVolumeMonitor *monitor)
   GList *l;
   GHalMount *mount;
   GHalVolume *volume;
+  const char *device_path;
   const char *mount_path;
   
   new_mounts = g_unix_mounts_get (NULL);
@@ -817,7 +921,7 @@ update_mounts (GHalVolumeMonitor *monitor)
       GUnixMountEntry *mount_entry = l->data;
       
       mount = find_mount_by_mount_path (monitor, g_unix_mount_get_mount_path (mount_entry));
-      //g_warning ("hal removing mount %s (%p)", g_unix_mount_get_device_path (mount_entry), mount);
+      /*g_warning ("hal removing mount %s (%p)", g_unix_mount_get_device_path (mount_entry), mount);*/
       if (mount)
 	{
 	  g_hal_mount_unmounted (mount);
@@ -832,10 +936,13 @@ update_mounts (GHalVolumeMonitor *monitor)
     {
       GUnixMountEntry *mount_entry = l->data;
 
+      device_path = g_unix_mount_get_device_path (mount_entry);
       mount_path = g_unix_mount_get_mount_path (mount_entry);
-      volume = g_hal_volume_monitor_lookup_volume_for_mount_path (monitor, mount_path);
+      volume = g_hal_volume_monitor_lookup_volume_for_device_path (monitor, device_path);
+      if (volume == NULL)
+        volume = g_hal_volume_monitor_lookup_volume_for_mount_path (monitor, mount_path);
 
-      //g_warning ("hal adding mount %s (vol %p)", g_unix_mount_get_device_path (mount_entry), volume);
+      /*g_warning ("hal adding mount %s (vol %p)", g_unix_mount_get_device_path (mount_entry), volume);*/
       mount = g_hal_mount_new (G_VOLUME_MONITOR (monitor), mount_entry, monitor->pool, volume);
       if (mount)
 	{
@@ -874,8 +981,8 @@ update_discs (GHalVolumeMonitor *monitor)
   new_optical_disc_devices = hal_pool_find_by_capability (monitor->pool, "volume.disc");
   for (l = new_optical_disc_devices; l != NULL; l = ll)
     {
-      ll = l->next;
       HalDevice *d = l->data;
+      ll = l->next;
       if (! (hal_device_get_property_bool (d, "volume.disc.is_blank") ||
              hal_device_get_property_bool (d, "volume.disc.has_audio")))
         {
@@ -896,7 +1003,7 @@ update_discs (GHalVolumeMonitor *monitor)
       HalDevice *d = l->data;
 
       udi = hal_device_get_udi (d);
-      //g_warning ("audio/blank disc removing %s", udi);
+      /*g_warning ("audio/blank disc removing %s", udi);*/
 
       mount = find_disc_mount_by_udi (monitor, udi);
       if (mount != NULL)
@@ -924,28 +1031,29 @@ update_discs (GHalVolumeMonitor *monitor)
       HalDevice *d = l->data;
 
       udi = hal_device_get_udi (d);
-      //g_warning ("audio/blank disc adding %s", udi);
+      /*g_warning ("audio/blank disc adding %s", udi);*/
 
       drive_udi = hal_device_get_property_string (d, "block.storage_device");
       drive = find_drive_by_udi (monitor, drive_udi);
       if (drive != NULL)
         {
+          mount = NULL;
           if (hal_device_get_property_bool (d, "volume.disc.is_blank"))
             {
-              volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, NULL, drive);
+              volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, NULL, FALSE, drive);
               if (volume != NULL)
                 {
-                  GFile *file;
-                  file = g_file_new_for_uri ("burn:///");
+                  GFile *root;
+                  root = g_file_new_for_uri ("burn:///");
                   mount = g_hal_mount_new_for_hal_device (G_VOLUME_MONITOR (monitor), 
                                                           d, 
-                                                          file,
+                                                          root,
                                                           NULL,
                                                           NULL,
                                                           TRUE,
                                                           monitor->pool,
                                                           volume);
-                  g_object_unref (file);
+                  g_object_unref (root);
                 }
             }
           else
@@ -961,7 +1069,7 @@ update_discs (GHalVolumeMonitor *monitor)
               g_free (device_basename);
               g_free (uri);
 
-              volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, foreign_mount_root, drive);
+              volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, foreign_mount_root, TRUE, drive);
               g_object_unref (foreign_mount_root);
               mount = NULL;
             }
@@ -987,8 +1095,96 @@ update_discs (GHalVolumeMonitor *monitor)
   monitor->last_optical_disc_devices = new_optical_disc_devices;
 }
 
+static void
+update_cameras (GHalVolumeMonitor *monitor)
+{
+#ifdef _WITH_GPHOTO2
+  GList *new_camera_devices;
+  GList *removed, *added;
+  GList *l, *ll;
+  GHalVolume *volume;
+  const char *udi;
+
+  new_camera_devices = hal_pool_find_by_capability (monitor->pool, "camera");
+  for (l = new_camera_devices; l != NULL; l = ll)
+    {
+      HalDevice *d = l->data;
+      ll = l->next;
+      /*g_warning ("got %s", hal_device_get_udi (d));*/
+      if (! hal_device_get_property_bool (d, "camera.libgphoto2.support"))
+        {
+          /*g_warning ("ignoring %s", hal_device_get_udi (d));*/
+          /* filter out everything that isn't supported by libgphoto2 */
+          new_camera_devices = g_list_delete_link (new_camera_devices, l);
+        }
+    }
+  g_list_foreach (new_camera_devices, (GFunc) g_object_ref, NULL);
+
+  new_camera_devices = g_list_sort (new_camera_devices, (GCompareFunc) hal_device_compare);
+  diff_sorted_lists (monitor->last_camera_devices,
+                     new_camera_devices, (GCompareFunc) hal_device_compare,
+                     &added, &removed);
+
+  for (l = removed; l != NULL; l = l->next)
+    {
+      HalDevice *d = l->data;
+
+      udi = hal_device_get_udi (d);
+      /*g_warning ("camera removing %s", udi);*/
+
+      volume = find_camera_volume_by_udi (monitor, udi);
+      if (volume != NULL)
+        {
+	  g_hal_volume_removed (volume);
+	  monitor->camera_volumes = g_list_remove (monitor->camera_volumes, volume);
+	  g_signal_emit_by_name (monitor, "volume_removed", volume);
+	  g_signal_emit_by_name (volume, "removed");
+	  g_object_unref (volume);
+        }
+    }
+  
+  for (l = added; l != NULL; l = l->next)
+    {
+      HalDevice *d = l->data;
+      char *uri;
+      GFile *foreign_mount_root;
+      int usb_bus_num;
+      int usb_device_num;
+
+      usb_bus_num = hal_device_get_property_int (d, "usb.bus_number");
+      usb_device_num = hal_device_get_property_int (d, "usb.linux.device_number");
+
+      uri = g_strdup_printf ("gphoto2://usb:%03d,%03d", usb_bus_num, usb_device_num);
+      /*g_warning ("uri is '%s'", uri);*/
+      foreign_mount_root = g_file_new_for_uri (uri);
+      g_free (uri);
+
+      udi = hal_device_get_udi (d);
+      /*g_warning ("camera adding %s", udi);*/
+
+      volume = g_hal_volume_new (G_VOLUME_MONITOR (monitor), d, monitor->pool, foreign_mount_root, TRUE, NULL);
+      g_object_unref (foreign_mount_root);
+      if (volume != NULL)
+        {
+          monitor->camera_volumes = g_list_prepend (monitor->camera_volumes, volume);
+          g_signal_emit_by_name (monitor, "volume_added", volume);
+        }
+    }
+
+  g_list_free (added);
+  g_list_free (removed);
+  g_list_foreach (monitor->last_camera_devices, (GFunc)g_object_unref, NULL);
+  g_list_free (monitor->last_camera_devices);
+  monitor->last_camera_devices = new_camera_devices;
+#endif
+}
+
 void 
 g_hal_volume_monitor_register (GIOModule *module)
 {
   g_hal_volume_monitor_register_type (G_TYPE_MODULE (module));
+  g_io_extension_point_implement (G_NATIVE_VOLUME_MONITOR_EXTENSION_POINT_NAME,
+				  G_TYPE_HAL_VOLUME_MONITOR,
+				  "hal",
+				  1);
 }
